"use strict";
exports.__esModule = true;
var Scope = /** @class */ (function () {
    function Scope(source, context) {
        this.localTypes = new Set();
        this.exportedTypes = new Set();
        this.localEnums = new Set();
        this.exportedEnums = new Set();
        this.source = source;
        this.context = context;
    }
    Scope.prototype.createError = function (msg, node) {
        var _a = this.source.getLineAndCharacterOfPosition(node.pos), line = _a.line, character = _a.character;
        var str = msg + ' ' + this.source.fileName + ':' + line + ',' + character;
        return new Error(str);
    };
    Scope.prototype.addLocalTypeName = function (name) {
        this.localTypes.add(name);
    };
    Scope.prototype.addExport = function (exported, local) {
        if (this.localTypes.has(local || exported)) {
            this.exportedTypes.add(exported);
        }
        if (this.localEnums.has(local || exported)) {
            this.exportedEnums.add(exported);
        }
    };
    Scope.prototype.addEnum = function (name) {
        this.localEnums.add(name);
    };
    return Scope;
}());
exports["default"] = Scope;
