"use strict";
exports.__esModule = true;
var fs_1 = require("fs");
var path_1 = require("path");
var tt = require("typescript");
var prettier_1 = require("prettier");
var Scope_1 = require("./Scope");
var transformStatement_1 = require("./transformStatement");
var TransformContext = /** @class */ (function () {
    function TransformContext() {
        this._cache = new Map();
    }
    TransformContext.prototype.transform = function (filename, src) {
        var trueFilename = path_1.resolve(filename);
        var cachedResult = this._cache.get(trueFilename);
        if (cachedResult) {
            return cachedResult;
        }
        if (src == null) {
            src = fs_1.readFileSync(trueFilename, 'utf8');
        }
        var sourceFile = tt.createSourceFile(trueFilename, src, tt.ScriptTarget.ES2017, false);
        var scope = new Scope_1["default"](sourceFile, this);
        var result = { code: '', scope: scope };
        this._cache.set(trueFilename, result);
        result.code = prettier_1.format('// @flow\n// Generated using flowgen2\n\n' + sourceFile.statements.map(function (statement) { return transformStatement_1["default"](statement, scope); }).join('\n'), {
            bracketSpacing: false,
            filepath: trueFilename.replace(/.ts$/, '.js'),
            parser: 'flow',
            singleQuote: true,
            trailingComma: 'all'
        });
        return result;
    };
    return TransformContext;
}());
exports["default"] = TransformContext;
