"use strict";
exports.__esModule = true;
var tt = require("typescript");
var declarationExports_1 = require("./declarationExports");
var transformExpressionWithTypeArguments_1 = require("./transformExpressionWithTypeArguments");
var transformIdentifier_1 = require("./transformIdentifier");
var transformClassElement_1 = require("./transformClassElement");
var transformTypeParameters_1 = require("./transformTypeParameters");
function transformHeritageClauses(heritageClauses, scope) {
    // SyntaxKind.ExtendsKeyword
    for (var _i = 0, heritageClauses_1 = heritageClauses; _i < heritageClauses_1.length; _i++) {
        var clause = heritageClauses_1[_i];
        if (clause.token === tt.SyntaxKind.ExtendsKeyword) {
            return ' extends ' + clause.types.map(function (type) { return transformExpressionWithTypeArguments_1["default"](type, scope); });
        }
    }
    // TODO: implements?
    return '';
}
function transformClassDeclaration(node, scope) {
    if (!node.name) {
        throw scope.createError('Expected class declaration to have a name', node);
    }
    var typeParameters = transformTypeParameters_1["default"](node.typeParameters, scope);
    var heritageClauses = node.heritageClauses ? transformHeritageClauses(node.heritageClauses, scope) : '';
    return "\n    declare class " + transformIdentifier_1["default"](node.name, scope) + typeParameters + " " + heritageClauses + " {\n      " + node.members.map(function (element) { return transformClassElement_1["default"](element, scope); }).join('') + "\n    }\n    " + declarationExports_1["default"](node, scope) + "\n  ";
}
exports["default"] = transformClassDeclaration;
