"use strict";
exports.__esModule = true;
var tt = require("typescript");
var transformIdentifier_1 = require("./transformIdentifier");
var aliases = {
    'NodeJS.ErrnoException': 'ErrnoError',
    'NodeJS.ReadableStream': 'stream$Readable',
    'NodeJS.WritableStream': 'stream$Writable',
    Partial: '$Shape',
    PromiseLike: 'Promise'
};
function resolveAlias(name) {
    if (typeof aliases[name] === 'string') {
        return aliases[name];
    }
    if (/^NodeJS\./.test(name)) {
        throw new Error("Unrecognised NodeJS entity name \"" + name + "\". Please submit a pull request to https://github.com/ForbesLindesay/flowgen2 to add the appropriate \"alias\" to \"transformEntityName.ts\"");
    }
    return name;
}
function transformEntityName(node, scope) {
    switch (node.kind) {
        case tt.SyntaxKind.Identifier:
            return resolveAlias(transformIdentifier_1["default"](node, scope));
        case tt.SyntaxKind.QualifiedName:
            return resolveAlias(transformEntityName(node.left, scope) +
                '.' +
                transformEntityName(node.right, scope));
    }
}
exports["default"] = transformEntityName;
