"use strict";
exports.__esModule = true;
var path_1 = require("path");
var fs_1 = require("fs");
var tt = require("typescript");
var resolve_1 = require("resolve");
var Scope_1 = require("./Scope");
var transformIdentifier_1 = require("./transformIdentifier");
var transformStringLiteral_1 = require("./transformStringLiteral");
// TODO: share code with transformImportEqualsDeclaration
var aliases = {
    // import { IncomingHttpHeaders } from 'http';
    http: {
        IncomingHttpHeaders: 'Object'
    }
};
function packageFilter(pkg) {
    if (pkg.types) {
        pkg.main = pkg.types;
    }
    return pkg;
}
function tryResolve(name, dirname) {
    try {
        var filename = resolve_1.sync(name, {
            basedir: dirname,
            extensions: ['.d.ts'],
            packageFilter: packageFilter
        });
        if (/\.js$/.test(filename)) {
            filename = filename.replace(/\.js$/, '.d.ts');
        }
        fs_1.statSync(filename);
        return filename;
    }
    catch (ex) {
        if (name[0] !== '.') {
            try {
                var filename = resolve_1.sync('@types/' + name, {
                    basedir: dirname,
                    extensions: ['.d.ts']
                });
                fs_1.statSync(filename);
                return filename;
            }
            catch (ex) { }
        }
    }
    return null;
}
function transformImportDeclaration(statement, scope) {
    if (!statement.importClause) {
        return '';
    }
    var moduleSpecifier = statement.moduleSpecifier;
    var moduleAliases = {};
    if (moduleSpecifier.text in aliases) {
        moduleAliases = aliases[moduleSpecifier.text];
    }
    var importedFileName = transformStringLiteral_1["default"](moduleSpecifier, scope);
    var filename = tryResolve(moduleSpecifier.text, path_1.dirname(scope.source.fileName));
    var importedScope = filename
        ? scope.context.transform(filename).scope
        : new Scope_1["default"](scope.source, scope.context);
    var result = [];
    var _a = statement.importClause, name = _a.name, namedBindings = _a.namedBindings;
    if (name) {
        var n = transformIdentifier_1["default"](name, scope);
        if (importedScope.exportedTypes.has('default')) {
            scope.addLocalTypeName(name.text);
            result.push("import type {defaultType as " + n + "} from " + importedFileName + ";");
        }
        else {
            if (importedScope.exportedEnums.has('default')) {
                scope.addEnum(name.text);
                result.push("import type {defaultType as " + n + "Type} from " + importedFileName + ";");
            }
            result.push("import " + n + " from " + importedFileName + ";");
        }
    }
    if (namedBindings) {
        switch (namedBindings.kind) {
            case tt.SyntaxKind.NamespaceImport:
                result.push("import * as " + transformIdentifier_1["default"](namedBindings.name, scope) + " from " + importedFileName + ";");
                break;
            case tt.SyntaxKind.NamedImports:
                var specifier = namedBindings;
                specifier.elements.forEach(function (importSpecifier) {
                    var local = importSpecifier.name.text;
                    var imported = (importSpecifier.propertyName || importSpecifier.name).text;
                    var specifierText = local === imported ? "{" + local + "}" : "{" + imported + " as " + local + "}";
                    if (imported in moduleAliases) {
                        result.push("type " + local + " = " + moduleAliases[imported] + ";");
                        scope.addLocalTypeName(local);
                    }
                    else if (importedScope.exportedTypes.has(imported)) {
                        scope.addLocalTypeName(local);
                        var specifierText_1 = local === imported ? "{" + local + "}" : "{" + imported + " as " + local + "}";
                        result.push("import type " + specifierText_1 + " from " + importedFileName + ";");
                    }
                    else {
                        if (importedScope.exportedEnums.has(imported)) {
                            scope.addEnum(local);
                            if (imported === local) {
                                result.push("import type {" + local + "Type} from " + importedFileName + ";");
                            }
                            else {
                                result.push("import type {" + imported + "Type as " + local + "Type} from " + importedFileName + ";");
                            }
                        }
                        result.push("import " + specifierText + " from " + importedFileName + ";");
                    }
                });
                break;
        }
    }
    return result.join('\n');
}
exports["default"] = transformImportDeclaration;
