"use strict";
exports.__esModule = true;
var tt = require("typescript");
var transformArrayType_1 = require("./transformArrayType");
var transformExpression_1 = require("./transformExpression");
var transformParenthesizedType_1 = require("./transformParenthesizedType");
var transformTypeLiteral_1 = require("./transformTypeLiteral");
var transformTypeReference_1 = require("./transformTypeReference");
var transformUnionType_1 = require("./transformUnionType");
var transformIntersectionType_1 = require("./transformIntersectionType");
var transformFunctionType_1 = require("./transformFunctionType");
var transformTupleType_1 = require("./transformTupleType");
var transformMappedType_1 = require("./transformMappedType");
var transformTypeOperator_1 = require("./transformTypeOperator");
var transformIndexedAccessType_1 = require("./transformIndexedAccessType");
function transformTypeNode(node, scope) {
    switch (node.kind) {
        case tt.SyntaxKind.LiteralType:
            return transformExpression_1["default"](node.literal, scope);
        case tt.SyntaxKind.TypeLiteral:
            return transformTypeLiteral_1["default"](node, scope);
        case tt.SyntaxKind.StringKeyword:
            return 'string';
        case tt.SyntaxKind.NumberKeyword:
            return 'number';
        case tt.SyntaxKind.AnyKeyword:
            return 'any';
        case tt.SyntaxKind.NullKeyword:
            return 'null';
        case tt.SyntaxKind.VoidKeyword:
            return 'void';
        case tt.SyntaxKind.UndefinedKeyword:
            return 'void';
        case tt.SyntaxKind.BooleanKeyword:
            return 'boolean';
        case tt.SyntaxKind.ThisType:
            return 'this';
        case tt.SyntaxKind.TypeReference:
            return transformTypeReference_1["default"](node, scope);
        case tt.SyntaxKind.UnionType:
            return transformUnionType_1["default"](node, scope);
        case tt.SyntaxKind.IntersectionType:
            return transformIntersectionType_1["default"](node, scope);
        case tt.SyntaxKind.ArrayType:
            return transformArrayType_1["default"](node, scope);
        case tt.SyntaxKind.FunctionType:
            return transformFunctionType_1["default"](node, scope);
        case tt.SyntaxKind.ParenthesizedType:
            return transformParenthesizedType_1["default"](node, scope);
        case tt.SyntaxKind.TupleType:
            return transformTupleType_1["default"](node, scope);
        case tt.SyntaxKind.MappedType:
            return transformMappedType_1["default"](node, scope);
        case tt.SyntaxKind.TypeOperator:
            return transformTypeOperator_1["default"](node, scope);
        case tt.SyntaxKind.IndexedAccessType:
            return transformIndexedAccessType_1["default"](node, scope);
        // TODO: support more cases
    }
    throw scope.createError('Unsupported type kind ' + tt.SyntaxKind[node.kind], node);
}
exports["default"] = transformTypeNode;
